﻿' •—————————————————————————————————————————————————————————————————————————————————————————————————————•
'                              National Institute of Standards and Technology
'                                          Biometric Clients Lab
' •—————————————————————————————————————————————————————————————————————————————————————————————————————•
'  File author(s):
'       Ross J. Micheals (ross.micheals@nist.gov)
'       Kevin Mangold (kevin.mangold@nist.gov)
'
' •—————————————————————————————————————————————————————————————————————————————————————————————————————•
' | NOTICE & DISCLAIMER                                                                                 |
' |                                                                                                     |
' | The research software provided on this web site (“software”) is provided by NIST as a public 		|
' | service. You may use, copy and distribute copies of the software in any medium, provided that you 	|
' | keep intact this entire notice. You may improve, modify and create derivative works of the software	|
' | or any portion of the software, and you may copy and distribute such modifications or works.  		|
' | Modified works should carry a notice stating that you changed the software and should note the date	|
' | and nature of any such change.  Please explicitly acknowledge the National Institute of Standards	|
' | and Technology as the source of the software.														|
' | 																									|
' | The software is expressly provided “AS IS.”  NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED, 	|
' | IN FACT OR ARISING BY OPERATION OF LAW, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF 		|
' | MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT AND DATA ACCURACY.  NIST 		|
' | NEITHER REPRESENTS NOR WARRANTS THAT THE OPERATION OF THE SOFTWARE WILL BE UNINTERRUPTED OR 		|
' | ERROR-FREE, OR THAT ANY DEFECTS WILL BE CORRECTED.  NIST DOES NOT WARRANT OR MAKE ANY 				|
' | REPRESENTATIONS REGARDING THE USE OF THE SOFTWARE OR THE RESULTS THEREOF, INCLUDING BUT NOT LIMITED	|
' | TO THE CORRECTNESS, ACCURACY, RELIABILITY, OR USEFULNESS OF THE SOFTWARE.							|
' | 																									|
' | You are solely responsible for determining the appropriateness of using and distributing the 		|
' | software and you assume all risks associated with its use, including but not limited to the risks	|
' | and costs of program errors, compliance with applicable laws, damage to or loss of data, programs	|
' | or equipment, and the unavailability or interruption of operation.  This software is not intended	|
' | to be used in any situation where a failure could cause risk of injury or damage to property.  The	|
' | software was developed by NIST employees.  NIST employee contributions are not subject to copyright	|
' | protection within the United States.  																|
' | 																									|
' | Specific hardware and software products identified in this open source project were used in order   |
' | to perform technology transfer and collaboration. In no case does such identification imply         |
' | recommendation or endorsement by the National Institute of Standards and Technology, nor            |
' | does it imply that the products and equipment identified are necessarily the best available for the |
' | purpose.                                                                                            |
' •—————————————————————————————————————————————————————————————————————————————————————————————————————•

Option Strict On
Option Infer On

Imports System.Runtime.Serialization
Imports System.ServiceModel
Imports System.ServiceModel.Web
Imports System.Threading
Imports System.Threading.Tasks

Imports Nist.Bcl.Wsbd


<TestClass()>
Public Class BaselineSensorServiceConfigurationTests
    Inherits BaselineSensorServiceTests


#Region " Get Configuration"

    <TestMethod(), TestCategory(GetConfiguration), TestCategory(InvalidId)> _
    Sub GettingConfigurationForAnUnregisterdIdGivesAnInvalidId()
        Dim client = CreateClient()
        Dim result = client.GetConfiguration(Guid.NewGuid.ToString).WsbdResult
        Assert.AreEqual(Status.InvalidId, result.Status)
    End Sub

    <TestMethod(), TestCategory(GetConfiguration), TestCategory(BadValue)> _
    Sub GettingConfigurationForAnUnparseableIdGivesABadValue()
        Dim client = CreateClient()
        Dim result = client.GetConfiguration("this_is_not_a_guid").WsbdResult
        Assert.AreEqual(Status.BadValue, result.Status)
    End Sub

    <TestMethod(), TestCategory(GetConfiguration), TestCategory(LockNotHeld)> _
    Sub GettingConfigurationRequiresALock()
        Dim client = CreateClient()
        Dim sessionId = client.Register().WsbdResult.SessionId.Value.ToString
        Dim result = client.GetConfiguration(sessionId).WsbdResult
        Assert.AreEqual(Status.LockNotHeld, result.Status)
    End Sub

    <TestMethod(), TestCategory(GetConfiguration), TestCategory(SensorFailure)>
    Sub GettingConfigurationCanResultInSensorFailure()
        BaselineSensorService.ForceSensorFailure = True
        Dim client = CreateClient()
        Dim sessionId = client.Register().WsbdResult.SessionId.Value.ToString
        client.Lock(sessionId)
        Dim result = client.GetConfiguration(sessionId).WsbdResult
        Assert.AreEqual(Status.SensorFailure, result.Status)
    End Sub

    <TestMethod(), TestCategory(GetConfiguration), TestCategory(InitializationRequired)>
    Sub GettingConfigurationMayRequireInitialization()
        BaselineSensorService.ForceInitializationRequired = True
        Dim client = CreateClient()
        Dim sessionId = client.Register().WsbdResult.SessionId.Value.ToString
        client.Lock(sessionId)
        Dim result = client.GetConfiguration(sessionId).WsbdResult
        Assert.AreEqual(Status.InitializationNeeded, result.Status)
    End Sub

    <TestMethod(), TestCategory(GetConfiguration), TestCategory(ConfigurationRequired)>
    Sub GettingConfigurationMayRequireConfiguration()
        BaselineSensorService.ForceConfigurationRequired = True
        Dim client = CreateClient()
        Dim sessionId = client.Register().WsbdResult.SessionId.Value.ToString
        client.Lock(sessionId)
        Dim result = client.GetConfiguration(sessionId).WsbdResult
        Assert.AreEqual(Status.ConfigurationNeeded, result.Status)
    End Sub

    <TestMethod(), TestCategory(GetConfiguration), TestCategory(Failure)>
    Sub GettingConfigurationCanResultInGeneralFailure()
        BaselineSensorService.ForceGeneralFailure = True
        Dim client = CreateClient()
        Dim sessionId = client.Register().WsbdResult.SessionId.Value.ToString
        client.Lock(sessionId)
        Dim result = client.GetConfiguration(sessionId).WsbdResult
        Assert.AreEqual(Status.Failure, result.Status)
    End Sub

    <TestMethod(), TestCategory(GetConfiguration), TestCategory(SensorTimeout)>
    Sub GettingConfigurationCanResultInSensorTimeout()
        BaselineSensorService.ForceSensorTimeout = True
        Dim client = CreateClient()
        Dim sessionId = client.Register().WsbdResult.SessionId.Value.ToString
        client.Lock(sessionId)
        Dim result = client.GetConfiguration(sessionId).WsbdResult
        Assert.AreEqual(Status.SensorTimeout, result.Status)
    End Sub

    <TestMethod(), TestCategory(GetConfiguration), TestCategory(LockHeldByAnother)>
    Sub CannotGetConfigurationWhenLockHeldByAnother()
        Dim client1 = CreateClient()
        Dim client2 = CreateClient()
        Dim session1 = client1.Register().WsbdResult.SessionId.Value.ToString
        Dim session2 = client2.Register().WsbdResult.SessionId.Value.ToString
        client1.Lock(session1)
        Dim result = client2.GetConfiguration(session2).WsbdResult
        Assert.AreEqual(Status.LockHeldByAnother, result.Status)
    End Sub

    <TestMethod(), TestCategory(GetConfiguration), TestCategory(Reentrant), TestCategory(Success), TestCategory(SensorBusy)>
    Sub WithSimultaneousCallsOnlyOneCallGetsConfigurationSuccessfully()

        Dim busyClients As Integer = 128

        '' ----------
        '' Begin test
        '' ----------

        BaselineSensorService.InitializationTime = 10000 'ms. 

        ' Make the timeout so long that the server never Cancels the operation
        ServiceInformation.GetInstance().InitializationTimeout = BaselineSensorService.InitializationTime * 1000

        Dim mainClient = CreateClient()
        Dim mainResult As Result


        Dim sessionId = mainClient.Register.WsbdResult.SessionId.Value.ToString
        mainClient.Lock(sessionId)
        Dim runMainClient As New Thread(Sub()
                                            mainResult = mainClient.GetConfiguration(sessionId).WsbdResult
                                        End Sub)

        ' Start a sensor operation and wait for the thread to really start
        runMainClient.Start()
        Threading.Thread.Sleep(500)


        For i As Integer = 0 To busyClients - 1
            Dim client = CreateClient()
            Dim result = client.GetConfiguration(sessionId).WsbdResult
            Assert.AreEqual(Status.SensorBusy, result.Status)
        Next

        runMainClient.Join()

    End Sub

    <TestMethod(), TestCategory(GetConfiguration), TestCategory(Cancel)>
    Sub GetConfigurationCanBeCanceledSuccessfully()

        ' Test parameters 
        Dim timeBeforeCancelation As Integer = 1000 ' ms

        '' Begin test

        Dim client = CreateClient()
        Dim infoResult As Result = Nothing
        Dim cancelResult As Result = Nothing


        Dim sessionId = client.Register.WsbdResult.SessionId.Value.ToString
        client.Lock(sessionId)

        Dim init As New Thread(Sub()
                                   infoResult = client.GetConfiguration(sessionId).WsbdResult
                               End Sub)

        Dim cancel As New Thread(Sub()
                                     Thread.Sleep(timeBeforeCancelation)
                                     cancelResult = client.Cancel(sessionId).WsbdResult
                                 End Sub)
        init.Start()
        cancel.Start()

        init.Join()
        cancel.Join()

        Assert.AreEqual(Status.Canceled, infoResult.Status)
        Assert.AreEqual(Status.Success, cancelResult.Status)

    End Sub

    <TestMethod(), TestCategory(GetConfiguration), TestCategory(CanceledWithSensorFailure)>
    Sub CancelingGetConfigurationCanCauseSensorFailure()

        ' Test parameters 
        Dim timeBeforeCancelation As Integer = 1000 ' ms

        '' Begin test

        Dim client = CreateClient()
        Dim infoResult As Result = Nothing
        Dim cancelResult As Result = Nothing

        BaselineSensorService.ForceSensorFailure = True

        Dim sessionId = client.Register.WsbdResult.SessionId.Value.ToString
        client.Lock(sessionId)

        Dim init As New Thread(Sub()
                                   infoResult = client.GetConfiguration(sessionId).WsbdResult
                               End Sub)

        Dim cancel As New Thread(Sub()
                                     Thread.Sleep(timeBeforeCancelation)
                                     cancelResult = client.Cancel(sessionId).WsbdResult
                                 End Sub)
        init.Start()
        cancel.Start()

        init.Join()
        cancel.Join()

        Assert.AreEqual(Status.CanceledWithSensorFailure, infoResult.Status)
        Assert.AreEqual(Status.Success, cancelResult.Status)

    End Sub


    <TestMethod(), TestCategory(GetConfiguration), TestCategory(Success)> _
    Sub GettingConfigurationCanSucceed()

        Dim configuration As New Dictionary
        configuration.Add("string1", "value")
        configuration.Add("integer1", 1)

        BaselineSensorService.Configuration = configuration

        Dim client = CreateClient()
        Dim sessionId = client.Register().WsbdResult.SessionId.Value.ToString
        client.Lock(sessionId)


        Dim result = client.GetConfiguration(sessionId).WsbdResult

        Assert.AreEqual(Status.Success, result.Status)
        Assert.IsNotNull(result.Metadata)
        Assert.AreEqual(result.Metadata.Count, 2)
        Assert.AreEqual(result.Metadata("string1"), "value")
        Assert.AreEqual(result.Metadata("integer1"), 1)
    End Sub


#End Region

#Region " Set Configuration "

    <TestMethod(), TestCategory(SetConfiguration), TestCategory(InvalidId)> _
    Sub SettingConfigurationForAnUnregisterdIdGivesAnInvalidId()
        Dim client = CreateClient()
        Dim result = client.SetConfiguration(Guid.NewGuid.ToString, New Configuration).WsbdResult
        Assert.AreEqual(Status.InvalidId, result.Status)
    End Sub

    <TestMethod(), TestCategory(SetConfiguration), TestCategory(BadValue)> _
    Sub SettingConfigurationForAnUnparseableIdGivesABadValue()
        Dim client = CreateClient()
        Dim result = client.SetConfiguration("this_is_not_a_guid", New Configuration).WsbdResult
        Assert.AreEqual(Status.BadValue, result.Status)
    End Sub

    <TestMethod(), TestCategory(SetConfiguration), TestCategory(LockNotHeld)> _
    Sub SettingConfigurationRequiresALock()
        Dim client = CreateClient()
        Dim sessionId = client.Register().WsbdResult.SessionId.Value.ToString
        Dim result = client.SetConfiguration(sessionId, New Configuration).WsbdResult
        Assert.AreEqual(Status.LockNotHeld, result.Status)
    End Sub

    <TestMethod(), TestCategory(SetConfiguration), TestCategory(SensorFailure)>
    Sub SettingConfigurationCanResultInSensorFailure()
        BaselineSensorService.ForceSensorFailure = True
        Dim client = CreateClient()
        Dim sessionId = client.Register().WsbdResult.SessionId.Value.ToString
        client.Lock(sessionId)
        Dim result = client.SetConfiguration(sessionId, New Configuration).WsbdResult
        Assert.AreEqual(Status.SensorFailure, result.Status)
    End Sub

    <TestMethod(), TestCategory(SetConfiguration), TestCategory(InitializationRequired)>
    Sub SettingConfigurationMayRequireInitialization()
        BaselineSensorService.ForceInitializationRequired = True
        Dim client = CreateClient()
        Dim sessionId = client.Register().WsbdResult.SessionId.Value.ToString
        client.Lock(sessionId)
        Dim result = client.SetConfiguration(sessionId, New Configuration).WsbdResult
        Assert.AreEqual(Status.InitializationNeeded, result.Status)
    End Sub

    <TestMethod(), TestCategory(SetConfiguration), TestCategory(Failure)>
    Sub SettingConfigurationCanResultInGeneralFailure()
        BaselineSensorService.ForceGeneralFailure = True
        Dim client = CreateClient()
        Dim sessionId = client.Register().WsbdResult.SessionId.Value.ToString
        client.Lock(sessionId)
        Dim result = client.SetConfiguration(sessionId, New Configuration).WsbdResult
        Assert.AreEqual(Status.Failure, result.Status)
    End Sub

    <TestMethod(), TestCategory(SetConfiguration), TestCategory(SensorTimeout)>
    Sub SettingConfigurationCanResultInSensorTimeout()
        BaselineSensorService.ForceSensorTimeout = True
        Dim client = CreateClient()
        Dim sessionId = client.Register().WsbdResult.SessionId.Value.ToString
        client.Lock(sessionId)
        Dim result = client.SetConfiguration(sessionId, New Configuration).WsbdResult
        Assert.AreEqual(Status.SensorTimeout, result.Status)
    End Sub


    <TestMethod(), TestCategory(SetConfiguration), TestCategory(Success)> _
    Sub SettingConfigurationCanSucceed()

        Dim configuration As New Configuration
        configuration.Add("string1", "value")
        configuration.Add("integer1", 1)

        Dim client = CreateClient()
        Dim sessionId = client.Register().WsbdResult.SessionId.Value.ToString
        client.Lock(sessionId)

        ' There should be no configuration before we call configure
        Assert.IsNull(BaselineSensorService.Configuration)

        Dim result = client.SetConfiguration(sessionId, configuration).WsbdResult

        ' After configuration, the service should be appropriately configured
        Assert.AreEqual(Status.Success, result.Status)
        Assert.AreEqual(BaselineSensorService.Configuration.Count, 2)
        Assert.AreEqual(BaselineSensorService.Configuration("string1"), "value")
        Assert.AreEqual(BaselineSensorService.Configuration("integer1"), 1)
    End Sub


    <TestMethod(), TestCategory(SetConfiguration), TestCategory(Unsupported)> _
    Sub SettingConfigurationCanYieldUnsupported()

        BaselineSensorService.ConfigurationValidator = Function(config As Dictionary) As Result
                                                           Dim r As New Result(Status.Unsupported)
                                                           r.BadFields = New StringArray
                                                           r.BadFields.Add("string1")
                                                           Return r
                                                       End Function


        Dim configuration As New Configuration
        configuration.Add("string1", "value")
        configuration.Add("integer1", 1)

        Dim client = CreateClient()
        Dim sessionId = client.Register.WsbdResult.SessionId.Value.ToString
        client.Lock(sessionId)
        Dim result = client.SetConfiguration(sessionId, configuration).WsbdResult
        Assert.AreEqual(Status.Unsupported, result.Status)

    End Sub

    <TestMethod(), TestCategory(SetConfiguration), TestCategory(NoSuchParameter)> _
    Sub SettingConfigurationCanYieldNoSuchParameter()

        BaselineSensorService.ConfigurationValidator = Function(config As Dictionary) As Result
                                                           Dim r As New Result(Status.NoSuchParameter)
                                                           r.BadFields = New StringArray
                                                           r.BadFields.Add("string1")
                                                           Return r
                                                       End Function


        Dim configuration As New Configuration
        configuration.Add("string1", "value")
        configuration.Add("integer1", 1)

        Dim client = CreateClient()
        Dim sessionId = client.Register.WsbdResult.SessionId.Value.ToString
        client.Lock(sessionId)
        Dim result = client.SetConfiguration(sessionId, configuration).WsbdResult
        Assert.AreEqual(Status.NoSuchParameter, result.Status)

    End Sub


    <TestMethod(), TestCategory(SetConfiguration), TestCategory(LockHeldByAnother)>
    Sub CannotSetConfigurationWhenLockHeldByAnother()
        Dim client1 = CreateClient()
        Dim client2 = CreateClient()
        Dim session1 = client1.Register().WsbdResult.SessionId.Value.ToString
        Dim session2 = client2.Register().WsbdResult.SessionId.Value.ToString
        client1.Lock(session1)
        Dim result = client2.SetConfiguration(session2, New Configuration).WsbdResult
        Assert.AreEqual(Status.LockHeldByAnother, result.Status)
    End Sub

    <TestMethod(), TestCategory(SetConfiguration), TestCategory(Reentrant), TestCategory(Success), TestCategory(SensorBusy)>
    Sub WithSimultaneousCallsOnlyOneCallSetsConfigurationSuccessfully()


        Dim busyClients As Integer = 128

        '' ----------
        '' Begin test
        '' ----------

        BaselineSensorService.InitializationTime = 10000 'ms. 

        ' Make the timeout so long that the server never cancels the operation
        ServiceInformation.GetInstance().InitializationTimeout = BaselineSensorService.InitializationTime * 1000

        Dim mainClient = CreateClient()
        Dim mainResult As Result

        Dim configuration As New Configuration
        configuration.Add("string1", "value")
        configuration.Add("integer1", 1)

        Dim sessionId = mainClient.Register.WsbdResult.SessionId.Value.ToString
        mainClient.Lock(sessionId)
        Dim runMainClient As New Thread(Sub()
                                            mainResult = mainClient.SetConfiguration(sessionId, configuration).WsbdResult
                                        End Sub)

        ' Start a sensor operation and wait for the thread to really start
        runMainClient.Start()
        Threading.Thread.Sleep(500)


        For i As Integer = 0 To busyClients - 1
            Dim client = CreateClient()
            Dim result = client.SetConfiguration(sessionId, configuration).WsbdResult
            Assert.AreEqual(Status.SensorBusy, result.Status)
        Next

        runMainClient.Join()

    End Sub

    <TestMethod(), TestCategory(SetConfiguration), TestCategory(Cancel)>
    Sub SetConfigurationCanBeCanceledSuccessfuly()

        ' Test parameters 
        Dim timeBeforeCancelation As Integer = 1000 ' ms

        '' Begin test

        Dim client = CreateClient()
        Dim infoResult As Result = Nothing
        Dim cancelResult As Result = Nothing


        Dim sessionId = client.Register.WsbdResult.SessionId.Value.ToString
        client.Lock(sessionId)

        Dim init As New Thread(Sub()
                                   infoResult = client.SetConfiguration(sessionId, New Configuration).WsbdResult
                               End Sub)

        Dim cancel As New Thread(Sub()
                                     Thread.Sleep(timeBeforeCancelation)
                                     cancelResult = client.Cancel(sessionId).WsbdResult
                                 End Sub)
        init.Start()
        cancel.Start()

        init.Join()
        cancel.Join()

        Assert.AreEqual(Status.Canceled, infoResult.Status)
        Assert.AreEqual(Status.Success, cancelResult.Status)

    End Sub


    <TestMethod(), TestCategory(SetConfiguration), TestCategory(CanceledWithSensorFailure)>
    Sub CancelingSetConfigurationCanCauseSensorFailure()

        ' Test parameters 
        Dim timeBeforeCancelation As Integer = 1000 ' ms

        '' Begin test

        Dim client = CreateClient()
        Dim infoResult As Result = Nothing
        Dim cancelResult As Result = Nothing

        BaselineSensorService.ForceSensorFailure = True

        Dim sessionId = client.Register.WsbdResult.SessionId.Value.ToString
        client.Lock(sessionId)

        Dim init As New Thread(Sub()
                                   infoResult = client.SetConfiguration(sessionId, New Configuration).WsbdResult
                               End Sub)

        Dim cancel As New Thread(Sub()
                                     Thread.Sleep(timeBeforeCancelation)
                                     cancelResult = client.Cancel(sessionId).WsbdResult
                                 End Sub)
        init.Start()
        cancel.Start()

        init.Join()
        cancel.Join()

        Assert.AreEqual(Status.CanceledWithSensorFailure, infoResult.Status)
        Assert.AreEqual(Status.Success, cancelResult.Status)

    End Sub

    <TestMethod(), TestCategory(SetConfiguration), ExpectedException(GetType(Net.WebException))>
    Sub SetConfigurationRequiresValidXML()

        Dim client = CreateClient()
        Dim sessionId = client.Register().WsbdResult.SessionId.Value.ToString
        client.Lock(sessionId)
        Dim result = client.SetConfiguration(sessionId, "this is not xml").WsbdResult
    End Sub

    <TestMethod()>
    Sub SetConfigurationExample()

        Dim client = CreateClient()
        Dim sessionId = client.Register().WsbdResult.SessionId.Value.ToString
        client.Lock(sessionId)

        Dim config = New Dictionary()
        config.Add("submodality", "leftMiddle")


        Dim configString = _
            <configuration xmlns:i="http://www.w3.org/2001/XMLSchema-instance" xmlns="urn:oid:2.16.840.1.101.3.9.3.1" xmlns:xs="http://www.w3.org/2001/XMLSchema">
                <item><key>submodality</key><value i:type="xs:string">leftMiddle</value></item></configuration>

        Dim result = client.SetConfiguration(sessionId, configString.ToString).WsbdResult
        Assert.AreEqual(Status.Success, result.Status)

    End Sub


    ''<TestMethod()>
    'Sub AAAAAAAAAAAAAAAAAA()
    '    Dim c As Dictionary = New Dictionary

    '    Dim payload As String

    '    Dim serializer As New DataContractSerializer(GetType(Dictionary))

    '    ' For debugging purposes, it's helpful to be able to see the payload as a string.
    '    Dim stringWriter As New System.IO.StringWriter
    '    Using xmlWriter = New System.Xml.XmlTextWriter(stringWriter)
    '        serializer.WriteObject(xmlWriter, DirectCast(c, Dictionary))
    '        payload = stringWriter.ToString
    '    End Using

    '    MsgBox(payload)
    'End Sub

#End Region

End Class
